\subsection{Examples}

> {-# LANGUAGE RankNTypes, FlexibleContexts #-} 
> 
> module Examples where
> import Control.Monad.State as State
> import Control.Monad.Reader as Reader
> import Control.Monad.Writer as Writer
> import Control.Monad.Identity as Identity
> import Data.Map as Map(Map)
> import SetBX 
> import StateTBX
> 
> 

Failure.

> 
> inv :: StateTBX Maybe Float Float Float
> inv = StateTBX get setL initL (gets (\ a -> 1 / a)) setR initR
>     where  setL a   = try put a
>            setR b   = try put (1/b)
>            try m a  = if a /= 0.0 then m a else lift Nothing
>            initL a  = if a /= 0.0 then Just a else Nothing
>            initR a  = if a /= 0.0 then Just (1/a) else Nothing

A generalization

> 
> divZeroBX ::  (Fractional a,Eq a,Monad m) => (forall x. m x) -> StateTBX m a a a
> divZeroBX divZero = StateTBX  get setL initL (gets (\a -> (1/a))) setR initR 
>   where
>     setL a        =   do {lift (guard (a /= 0)); put a}
>     setR b        =   do {lift (guard (b /= 0)); put (1/b)}
>     initL a       =   do {guard (a /= 0); return a}
>     initR b       =   do {guard (b /= 0); return (1/b)}
>     guard b       =   if b then return () else divZero

Uses |readS| to trap errors

> readSome ::  (Read a, Show a) => 
>              StateTBX Maybe (a, String) a String
> readSome = StateTBX (gets fst) setL initL (gets snd) setR initR
>   where  setL a' = put (a',show a')
>          setR b' = do  (_,b) <- get
>                        if b /= b' then return () 
>                        else case reads b of
>                               ((a',""):_) -> put(a',b) 
>                               _ -> lift Nothing
>          initL a = return (a, show a)
>          initR b = do  a <- choices [a|(a,"") <- reads b]
>                        return (a,b)
>          choices [] = mzero
>          choices (a:as) = return a `mplus` choices as

A generalization

> readableBX ::  (Read a, Show a,MonadPlus m) => 
>                StateTBX m (a,String) a String
> readableBX = StateTBX getA setA initA getB setB initB
>   where  getA     = do {(a,s) <- get; return a}
>          getB     = do {(a,s) <- get; return s}
>          setA a'  = put (a',show a')
>          setB b'  = do  (_,b) <- get
>                         case ((b == b'), reads b) of
>                           (True,_) -> return ()
>                           (_, ( a',""):_) -> put (a',b)
>                           _ -> lift mzero
>          initA a = return (a,show a)
>          initB b = case reads b of
>                      (a,""):_ -> return (a,b)
>                      _ -> mzero

The JTL example from the paper (\Example~\ref{ex:nondeterminism})

> nondetBX ::  (a->b->Bool) -> (a->[b]) -> (b->[a]) -> StateTBX [] (a,b) a b
> nondetBX ok bs as = StateTBX  getL setL initL getR setR initR where
>   getL     = do { (a,b) <- get ; return a }
>   getR     = do { (a,b) <- get ; return b }
>   setL a'  = do  (a,b) <- get 
>                  if ok a' b then put (a',b) else
>                    do { b' <- lift (bs a') ; put (a',b') }
>   setR b'  = do  (a,b) <- get 
>                  if ok a b' then put (a,b') else
>                    do { a' <- lift (as b') ; put (a',b') }
>   initL a  = do {b <- bs a; return (a,b)}
>   initR b  = do {a <- as b; return (a,b)}

Switching between two lenses on the same state space, based on a boolean flag

> switchBX :: MonadReader Bool m =>  StateTBX m s a b -> 
>                                    StateTBX m s a b -> 
>                                    StateTBX m s a b
> switchBX bx1 bx2 = StateTBX getA setA initA getB setB initB
>   where getA          =  switch (getl bx1) (getl bx2)
>         getB          =  switch (getr bx2) (getr bx2)
>         setA a        =  switch (setl bx1 a) (setl bx2 a)
>         setB b        =  switch (setr bx1 b) (setr bx2 b)
>         initA a       =  switch (initl bx1 a) (initl bx2 a)
>         initB b       =  switch (initr bx1 b) (initr bx2 b)
>         switch m1 m2  =  do {b <- ask; if b then m1 else m2}

Generalized version

> switchBX' :: MonadReader c m =>  (c -> StateTBX m s a b) -> 
>                                  StateTBX m s a b
> switchBX' f = StateTBX getA setA initA getB setB initB
>   where getA          =  switch getl
>         getB          =  switch getr
>         setA a        =  switch (\bx -> setl bx a)
>         setB b        =  switch (\bx -> setr bx b)
>         initA a       =  switch (\bx -> initl bx a)
>         initB b       =  switch (\bx -> initr bx b)
>         switch op     =  do {c <- ask; op (f c)}

Logging BX: writes the sequence of sets

> loggingBX ::  (Eq a, Eq b, MonadWriter (Either a b) m) => 
>               StateTBX m s a b -> StateTBX m s a b
> loggingBX bx = StateTBX (getl bx) setA (initl bx) (getr bx) setB  (initr bx)
>   where setA a' = do  a <- getl bx
>                       if a /= a' then tell (Left a') else return ()
>                       setl bx a'
>         setB b' = do  b <- getr bx
>                       if b /= b' then tell (Right b') else return ()
>                       setr bx b'

I/O: user interaction

> interactiveBX ::  (Read a, Read b) => 
>                   (a -> b -> Bool) -> StateTBX IO (a,b) a b
> interactiveBX r = StateTBX getA setA initA getB setB initB
>   where getA      =  do {(a,b) <- get; return a}
>         setA a'   =  do {(a,b) <- get; fixB a' b}
>         fixA a b  =  if    r a b 
>                      then  put(a,b)
>                      else  do {a' <- lift (initA b); fixA a' b}
>         initA b   =  do  print "Please restore consistency:"
>                          str <- getLine 
>                          return (read str)
>         getB      =  do {(a,b) <- get; return b}
>         setB b'   =  do {(a,b) <- get; fixA a b'}
>         fixB a b  =  if    r a b 
>                      then  put(a,b)
>                      else  do {b' <- lift (initB b); fixA a b'}
>         initB a   =  do  print "Please restore consistency:"
>                          str <- getLine 
>                          return (read str) 

and signalling changes

> signalBX ::  (Eq a, Eq b, Monad m) => 
>              (a -> m ()) -> (b -> m ()) -> 
>              StateTBX m s a b -> StateTBX m s a b
> signalBX sigA sigB t = StateTBX  (getl t) setL (initl t) 
>                                  (getr t) setR (initr t) where
>   setL a'   = do {  a <- getl t ; setl t a';
>                     lift (if a /= a' then sigA a' else return ()) }
>   setR b'   = do {  b <- getr t ; setr t b' ;
>                     lift (if b /= b' then sigB b' else return ()) }
>
> alertBX :: (Eq a, Eq b) => StateTBX IO s a b -> StateTBX IO s a b
> alertBX = signalBX  (\_ -> putStrLn "Left")
>                     (\_ -> putStrLn "Right")

where

> fst3 (a,_,_) = a
> snd3 (_,a,_) = a
> thd3 (_,_,a) = a

Model-transformation-by-example (\Example~\ref{ex:dynamic} from the paper)

> dynamicBX' ::  (Eq aT, Eq bT, Monad tT) => 
>                (aT -> bT -> tT bT) -> (aT -> bT -> tT aT) -> 
>                StateTBX tT ((aT,bT),[((aT,bT),bT)],[((aT,bT),aT)]) aT bT
> dynamicBX' f g = StateTBX  (gets (fst.fst3)) setL undefined 
>                            (gets (snd.fst3)) setR undefined
>   where  setL a' = do  ((a,b),fs,bs) <- get
>                        b' <- case lookup (a',b) fs of
>                           Just b'  -> return b'
>                           Nothing  -> lift (f a' b)
>                        put (  (a',b'), ((a',b),b') : ((a',b'),b') : fs, ((a',b'),a') :bs)
>          setR b' = do  ((a,b),fs,bs) <- get
>                        a' <- case lookup (a,b') bs of
>                           Just a'  -> return a'
>                           Nothing  -> lift (g a b')
>                        put (  (a',b'), ((a',b'),b') : fs, ((a,b'),a') : ((a',b'),a') : bs)




Some test cases
 
> l0 ::  Monad m => b -> c -> 
>        StateTBX m ((a,b), (c,a)) (a,b) (c,a) 
> l0 b c = fstBX b `compBX` coBX(sndBX c)
> 
> l = l0 "b" "c"
> 
> 
> foo = runl l   (do  setr l ("x","y")
>                     (a,b) <- getl l
>                     lift (print a)
>                     lift (print b)
>                     setl l ("z","w")
>                     (c,d) <- getr l
>                     lift (print c)
>                     lift (print d)) ("a","b")

> 
> l' :: (Read a, Ord a) => StateTBX IO (a,a) a a
> l' = interactiveBX (<)
> 
> bar = runStateT  (do  setr l' "abc"
>                       a <- getl l'
>                       lift (print a)
>                       setl l' "def"
>                       b <- getr l'
>                       lift (print b))
>                  ("abc","xyz")

> baz = let bx = (divZeroBX (fail "divZero")) 
>       in  runl bx   (do  setr bx 17.0
>                          a <- getl bx
>                          lift (print a)
>                          setl bx 42.0
>                          a <- getr bx
>                          lift (print a)
>                          setl bx 0.0
>                          lift (print "foo")) 1.0
                         
> xyzzy = let bx = listBX (divZeroBX (fail "divZero"))
>         in runl  bx (do  b <- getr bx
>                          lift (print b)
>                          setr bx [5.0,6.0,7.0,8.0]
>                          a <- getl bx
>                          lift (print a)) 
>                  [1.0,2.0,3.0]
> 
