\subsection{Symmetric Lenses}

> module SLens where
> import Lens
> import RelBX

Symmetric lenses (with explicit points) 
 
> data SLens c a b = SLens {  putr :: (a,c) -> (b,c), 
>                             putl :: (b,c) -> (a,c),
>                             missing :: c}

Dual

> dualSL sl = SLens (putl sl) (putr sl) missing

From asymmetric lenses

> lens2symlens :: Lens a b -> SLens (Maybe a) a b
> lens2symlens l = SLens putr putl Nothing
>   where  putr (a,_)    =  (view l a, Just a)
>          putl (b',ma)  =  let a' =  case ma of 
>                                        Nothing -> create l b'
>                                        a -> update l a' b' 
>                           in (create l b',Just a')

From relational bx

> rel2symlens ::  (Pointed a, Pointed b) => RelBX a b -> SLens (a,b) a b
> rel2symlens bx = SLens  (\(a',(a,b)) ->  let b' = fwd bx a' b 
>                                          in (b',(a',b')))
>                         (\(b',(a,b)) ->  let a' = bwd bx a b' 
>                                          in (a',(a',b')))
>                         (point,point)

To asymmetric lens, on the left...
 
> symlens2lensL :: SLens c a b -> Lens (a,b,c) a
> symlens2lensL sl = Lens  (\(a,b,c) -> a)
>                          (\(_,_,c) -> fixup c)
>                          (fixup (missing sl))
>   where fixup c a' = let (b',c') = putr sl (a',c) in (a',b',c')

...and on the right
 
> symlens2lensR :: SLens c a b -> Lens (a,b,c) b
> symlens2lensR sl = Lens  (\(a,b,c) -> b)
>                          (\ (_,_,c) -> fixup c)
>                          (fixup (missing sl))
>   where fixup c b' = let (a',c') = putl sl (b',c) in (a',b',c')

Spans and cospans: used to simplify some definitions.

> type Span c y1 x y2 = (c x y1, c x y2)
> type Cospan c y1 z y2 = (c y1 z, c y2 z)

To a span

> symlens2lensSpan :: SLens c a b -> Span Lens a (a,b,c) b
> symlens2lensSpan sl = (symlens2lensL sl, symlens2lensR sl)

From a span

> lensSpan2symlens :: Span Lens a c b -> SLens (Maybe c) a b
> lensSpan2symlens (l1,l2) = SLens  (\(a,mc) -> 
>                                     let c' = case mc of  Nothing -> create l1 a
>                                                          Just c -> update l1 c a 
>                                     in (view l2 c',Just c') )
>                                   (\(b,mc) -> 
>                                     let c' = case mc of  Nothing -> create l2 b
>                                                          Just c -> update l2 c b 
>                                     in (view l1 c',Just c') )
>                                   Nothing
> 
