\subsection{Monadic Symmetric Lenses}

> module SMLens where
> import MLens

Symmetric lenses (with explicit points) 
 
> data SMLens m c a b = SMLens {  mputr :: (a,c) -> m (b,c), 
>                                 mputl :: (b,c) -> m (a,c),
>                                 mmissing :: c}

Dual

> dualSL sl = SMLens (mputl sl) (mputr sl) mmissing

To asymmetric MLens, on the left...
 
> symMLens2MLensL ::  Monad m => SMLens m c a b -> MLens m (a,b,c) a
> symMLens2MLensL sl = MLens  (\(a,b,c) -> a)
>                             (\(_,_,c) -> fixup c)
>                             (fixup (mmissing sl))
>   where fixup c a' = do {  (b',c') <- mputr sl (a',c); return (a',b',c')}

...and on the right
 
> symMLens2MLensR ::  Monad m => SMLens m c a b -> MLens m (a,b,c) b
> symMLens2MLensR sl = MLens  (\(a,b,c) -> b)
>                             (\ (_,_,c) -> fixup c)
>                             (fixup (mmissing sl))
>   where fixup c b' = do {  (a',c') <- mputl sl (b',c); return (a',b',c')}

Spans and cospans: used to simplify some definitions.

> type Span c y1 x y2 = (c x y1, c x y2)
> type Cospan c y1 z y2 = (c y1 z, c y2 z)

To a span

> symlens2lensSpan ::  Monad m => SMLens m c a b -> Span (MLens m) a (a,b,c) b
> symlens2lensSpan sl = (  symMLens2MLensL sl, symMLens2MLensR sl)

and from a span

> lensSpan2symlens ::  Monad m => Span (MLens m) a c b -> SMLens m (Maybe c) a b
> lensSpan2symlens (l1,l2) 
>   = SMLens  (\(a,mc) ->  do  c' <- case mc of  Nothing -> mcreate l1 a
>                                                Just c -> mupdate l1 c a 
>                              return (mview l2 c',Just c') )
>             (\(b,mc) ->  do  c' <- case mc of  Nothing -> mcreate l2 b
>                                                Just c -> mupdate l2 c b 
>                              return (mview l1 c',Just c') )
>             Nothing

Composition (naive)

> compSMLens :: Monad m =>  SMLens m c1 a b -> SMLens m c2 b c -> SMLens m (c1,c2) a c
> compSMLens sl1 sl2 = SMLens mputR mputL mMissing where
>   mputR (a,(c1,c2))  = do  (b,c1') <- mputr sl1 (a,c1)
>                            (c,c2') <- mputr sl2 (b,c2)
>                            return (c,(c1',c2'))
>   mputL (c,(c1,c2))  = do  (b,c2') <- mputl sl2 (c,c2)
>                            (a,c1') <- mputl sl1 (b,c1)
>                            return (a,(c1',c2'))
>   mMissing           = (mmissing sl1, mmissing sl2) 
