\subsection{SetBX}

> {-# LANGUAGE RankNTypes, ImpredicativeTypes #-} 
> module SetBX where
> import Control.Monad.State as State
> import Control.Monad.Reader as Reader
> 
> 
> data SetBX m a b = SetBX {
>      mgetl :: m a,
>      msetl :: a -> m (),
>      mgetr :: m b, 
>      msetr :: b -> m ()
> }    
> 
> 
> mputlr :: Monad m => SetBX m a b -> a -> m b
> mputlr bx a = msetl bx a >> mgetr bx
> 
> mputrl :: Monad m => SetBX m a b -> b -> m a
> mputrl bx b = msetr bx b >> mgetl bx
> 
> 
> 

Identity.
 
> idMBX :: MonadState a m => SetBX m a a
> idMBX = SetBX get put get put

Duality.

> coMBX :: SetBX m a b -> SetBX m b a
> coMBX bx = SetBX (mgetr bx) (msetr bx) (mgetl bx) (msetl bx)
 
Natural transformations
 
> type NT f g = forall a. f a -> g a
> 
> type NTSpan g1 f g2 = (NT f g1, NT f g2)
> 
> type NTCospan f1 g f2 = (NT f1 g, NT f2 g)
> 
> type NTSquare f g1 g2 h = (NTSpan g1 f g2, NTCospan g1 h g2)
 
Composition

> compMBX ::  (NTCospan m1 n m2) -> SetBX m1 a b -> SetBX m2 b c -> 
>              SetBX n a c
> compMBX (phi,psi) bx1 bx2 = SetBX  (phi (mgetl bx1))
>                                    (\a -> phi (msetl bx1 a))
>                                    (psi (mgetr bx2))
>                                    (\a -> psi (msetr bx2 a))
 
Variant, assuming monad morphisms |l| and |r| 
 
> compMBX' :: (Monad m1, Monad m2, Monad n) =>  
>                        (NTCospan m1 n m2) -> 
>                        SetBX m1 a b -> SetBX m2 b c -> SetBX n a c
> compMBX' (l, r) bx1 bx2 = 
>        SetBX     (l (mgetl bx1))
>                  (\a -> do {  b <- l (do {msetl bx1 a;mgetr bx1}); 
>                               r (msetl bx2 b)})
>                  (r (mgetr bx2))
>                  (\c -> do {  b <- r (do {msetr bx2 c; mgetl bx2}); 
>                               l (msetr bx1 b)})
