\subsection{StateTBX}

> {-# LANGUAGE RankNTypes, FlexibleContexts #-} 
> 
> module StateTBX where
> import Control.Monad.State as State
> import Control.Monad.Identity as Identity
> import SetBX 
> import Iso
> import Lens
> import RelBX
> import SLens as SLens
> import MLens as MLens
> import SMLens as SMLens

The interface

> data StateTBX m s a b = StateTBX {
>      getl :: StateT s m a,
>      setl :: a -> StateT s m (),
>      initl :: a -> m s,
>      getr :: StateT s m b, 
>      setr :: b -> StateT s m (),
>      initr :: b -> m s
> }    

Variations on initialisation

> init2run ::  Monad m => (a -> m s) -> StateT s m x -> a -> m (x,s)
> init2run init m a = do {s <- init a; runStateT m s}
>
> runl ::  Monad m => StateTBX m s a b -> StateT s m x -> a -> m (x,s)
> runl bx = init2run (initl bx)
>
> runr ::  Monad m => StateTBX m s a b -> StateT s m x -> b -> m (x,s)
> runr bx = init2run (initr bx)
>
> run2init ::  Monad m => (forall x. StateT s m x -> a -> m (x,s)) -> a -> m s
> run2init run a = do {((),s) <- run (return ()) a; return s}

An alternative `PutBX' or push--pull style

> putLR :: Monad m => StateTBX m s a b -> a -> StateT s m b
> putLR bx a = do {setl bx a; getr bx}
>
> putRL :: Monad m => StateTBX m s a b -> b -> StateT s m a
> putRL bx b = do {setr bx b; getl bx}

Identity is easy

> idBX :: Monad m => StateTBX m a a a
> idBX = StateTBX get put return get put  return
 
Duality

> coBX :: StateTBX m s a b -> StateTBX m s b a
> coBX bx = StateTBX  (getr bx) (setr bx) (initr bx)
>                     (getl bx) (setl bx) (initl bx)
> 
 
Monad morphisms injecting |StateT s m| (respectively |StateT t m|) into
|StateT (s,t) m|. 

> left :: Monad m => StateT s m a -> StateT (s,t) m a
> left ma = do {  (s,t) <- get;
>                 (a,s') <- lift (runStateT ma s);
>                 put(s',t);
>                 return a}
> 
> right :: Monad m => StateT t m a -> StateT (s,t) m a
> right ma = do {  (s,t) <- get;
>                  (a,t') <- lift (runStateT ma t);
>                  put (s,t'); 
>                  return a }

Composition:
given |l :: StateTBX m s a b| and |l' :: StateTBX m s b c|,
we want
\begin{spec}
compBX l l' :: StateTBX m (s,t) a c
\end{spec}
satisfying the monad and bx laws

> thetaM :: Monad m => MLens m s v -> NT (StateT v m) (StateT s m)
> thetaM l m = do    s <- get 
>                    let v = mview l s
>                    (a,v') <- lift (runStateT m v) 
>                    s' <- lift (mupdate l s v') 
>                    put s' 
>                    return a 

The m-lenses induced by two composable bxs.

> mlensL :: Monad m =>  StateTBX m s1 a b -> 
>                       StateTBX m s2 b c -> 
>                       MLens m (s1,s2) s1
> mlensL bx1 bx2 = MLens view update create where
>         view (s1,s2)        = s1
>         update (s1,s2) s1'  = do  b <- evalStateT (getr bx1) s1'
>                                   s2' <- execStateT (setl bx2 b) s2
>                                   return (s1',s2')
>         create s1           = do  b <- evalStateT (getr bx1) s1
>                                   s2 <- initl bx2 b
>                                   return (s1,s2)
>
> mlensR :: Monad m =>  StateTBX m s1 a b -> 
>                       StateTBX m s2 b c -> 
>                       MLens m (s1,s2) s2
> mlensR bx1 bx2 = MLens view update create where
>         view (s1,s2)         = s2
>         update (s1,s2) s2'   = do  (b,_) <- runStateT (getl bx2) s2'
>                                    ((),s1') <- runStateT (setr bx1 b) s1
>                                    return (s1',s2')
>         create s2            = do  b <- evalStateT (getl bx2) s2
>                                    s1 <- initr bx1 b
>                                    return (s1,s2)

Composition in terms of m-lenses

> compBX :: (Monad m) =>  StateTBX m s1 a b -> 
>                         StateTBX m s2 b c -> 
>                         StateTBX m (s1,s2) a c
> compBX bx1 bx2 = 
>                  StateTBX  (phi (getl bx1)) (phi . (setl bx1))
>                            (\a -> do  (b,s) <- runl bx1 (getr bx1) a
>                                       t <- initl bx2 b
>                                       return (s,t))
>                            (psi (getr bx2)) (psi . (setr bx2))
>                            (\c -> do  (b,t) <- runr bx2 (getl bx2) c
>                                       s <- initr bx1 b              
>                                       return (s,t))
>        where  phi = thetaM (mlensL bx1 bx2)
>               psi = thetaM (mlensR bx1 bx2)

Alternative definition using |left| and |right|

> compBX' :: (Monad m) =>  StateTBX m s a b -> 
>                          StateTBX m t b c -> 
>                          StateTBX m (s,t) a c
> compBX' bx1 bx2 = 
>        StateTBX  (left (getl bx1))
>                  (\a -> do  b <- left (setl bx1 a >> getr bx1)
>                             right (setl bx2 b))
>                  (\a -> do  (b,s) <- runl bx1 (getr bx1) a
>                             t <- initl bx2 b
>                             return (s,t))
>                  (right (getr bx2))
>                  (\c -> do  b <- right (setr bx2 c >> getl bx2)
>                             left (setr bx1 b))
>                  (\c -> do  (b,t) <- runr bx2 (getl bx2) c
>                             s <- initr bx1 b              
>                             return (s,t))

Direct definition

> compBX0 :: (Monad m) =>  StateTBX m s a b -> 
>                          StateTBX m t b c -> 
>                          StateTBX m (s,t) a c
> compBX0 bx1 bx2 = 
>         StateTBX (do {(s,t) <- get; lift (evalStateT (getl bx1) s)})
>                  (\a -> do  (s,t) <- get
>                             s' <- lift (execStateT (setl bx1 a) s)
>                             b' <- lift (evalStateT (getr bx1) s')
>                             t' <- lift (execStateT (setl bx2 b') t)
>                             put (s',t'))
>                  (\a -> do  (b,s) <- runl bx1 (getr bx1) a
>                             t <- initl bx2 b
>                             return (s,t))
>                  (do {(s,t) <- get; lift (evalStateT (getr bx2) t)})
>                  (\c -> do  (s,t) <- get
>                             t' <- lift(execStateT (setr bx2 c) t)
>                             b' <- lift(evalStateT (getl bx2) t')
>                             s' <- lift(execStateT (setr bx1 b') s)
>                             put (s',t'))
>                  (\c -> do  (b,t) <- runr bx2 (getl bx2) c
>                             s <- initr bx1 b              
>                             return (s,t))
 
Isomorphisms
 
> iso2BX :: Monad m => Iso a b -> StateTBX m a a b
> iso2BX iso = StateTBX  get put return
>                        (do {a <- get; return (to iso a)})
>                        (\b -> do {put (from iso b)})
>                        (\b -> return (from iso b))
>
> assocBX :: Monad m =>  StateTBX m ((a,b),c) ((a,b),c) (a,(b,c))
> assocBX = iso2BX assocIso
>
> swapBX :: Monad m => StateTBX m (a,b) (a,b) (b,a)
> swapBX = iso2BX swapIso
>
> unitlBX :: Monad m => StateTBX m a a ((),a)
> unitlBX = iso2BX unitlIso
>
> unitrBX :: Monad m => StateTBX m a a (a,())
> unitrBX = iso2BX unitrIso

Lenses

> lens2BX :: Monad m => Lens a b -> StateTBX m a a b
> lens2BX l = StateTBX  get put return
>                       (do {a <- get; return (view l a)})
>                       (\b -> do {a <- get; put (update l a b)})
>                       (\b -> return (create l b))
>
> lensSpan2BX :: Monad m =>  Lens c a -> Lens c b -> StateTBX m c a b
> lensSpan2BX l1 l2 = StateTBX  (do  c <- get
>                                    return (view l1 c))
>                               (\a -> do  c <- get
>                                          put (update l1 c a))
>                               (\a -> return (create l1 a))
>                               (do  c <- get
>                                    return (view l2 c))
>                               (\b -> do  c <- get
>                                          put (update l2 c b))
>                               (\b -> return (create l2 b))

Monadic lenses

> mlens2BX :: Monad m => MLens m a b -> StateTBX m a a b
> mlens2BX l = StateTBX  get put return view upd create where
>   view              = gets (mview l)
>   upd b             = do {  a <- get; 
>                             a' <- lift (mupdate l a b); 
>                             put a'}
>   create b          = mcreate l b
>
> mlensSpan2BX :: Monad m =>  MLens m c a -> MLens m c b -> 
>                             StateTBX m c a b
> mlensSpan2BX l1 l2 = StateTBX  viewL updL createL
>                                viewR updR createR where
>   viewL         = gets (mview l1)
>   updL a        = do  c <- get
>                       c' <- lift (mupdate l1 c a)
>                       put c'
>   createL a     = mcreate l1 a
>   viewR         = gets (mview l2)
>   updR b        = do  c <- get
>                       c' <- lift (mupdate l2 c b)
>                       put c'
>   createR b     = mcreate l2 b

Relational bxs. 

> rel2BX ::  (Monad m,Pointed a,Pointed b) => 
>            RelBX a b -> StateTBX m (a,b) a b
> rel2BX bx = StateTBX (do {(a,b) <- get; return a})
>                      (\a' -> do {(a,b) <- get; put (a',fwd bx a' b)})
>                      (\a -> return (a,point))                     
>                      (do {(a,b) <- get; return b})
>                      (\b' -> do {(a,b) <- get; put (bwd bx a b',b')})
>                      (\b -> return (point,b))

Symmetric lenses 

> symlens2bx :: Monad m =>  SLens c a b -> StateTBX m (a,b,c) a b
> symlens2bx l = StateTBX  (do  (a,b,c) <- get
>                               return a)
>                          (\a' -> do  (a,b,c) <- get
>                                      let (b',c') = putr l (a',c)
>                                      put (a',b',c'))
>                          (\a -> do   let (b,c) = putr l (a,missing l)
>                                      return (a,b,c))
>                          (do  (a,b,c) <- get
>                               return (b))
>                          (\b' -> do  (a,b,c) <- get
>                                      let (a',c') = putl l (b',c)
>                                      put (a',b',c'))
>                          (\b -> do   let (a,c) = putl l (b,missing l) 
>                                      return (a,b,c))
>
> bx2symlens :: StateTBX Identity c a b -> SLens (Maybe c) a b
> bx2symlens bx = SLens  (\(a,mc) -> 
>                           let m = (setl bx a >> getr bx) in
>                           let (b',c') = 
>                                 case mc of 
>                                   Nothing -> runIdentity (runl bx m a);
>                                   Just c -> runIdentity (runStateT m c)
>                           in (b',Just c'))
>                        (\(b,mc) -> 
>                           let m = (setr bx b >> getl bx) in
>                           let (a',c') = 
>                                 case mc of 
>                                   Nothing -> runIdentity (runr bx m b);
>                                   Just c -> runIdentity (runStateT m c)
>                           in (a',Just c'))
>                        Nothing

Monadic symmetric lenses

> symMLens2bx :: Monad m =>  SMLens m c a b -> StateTBX m (a,b,c) a b
> symMLens2bx l = StateTBX  (do  (a,b,c) <- get
>                                return a)
>                           (\a' -> do  (a,b,c) <- get
>                                       (b',c') <-lift (mputr l (a',c))
>                                       put (a',b',c'))
>                           (\a -> do   (b,c) <- mputr l (a,mmissing l)
>                                       return (a,b,c))
>                           (do  (a,b,c) <- get
>                                return b)
>                           (\b' -> do  (a,b,c) <- get
>                                       (a',c') <- lift (mputl l (b',c))
>                                       put (a',b',c'))
>                           (\b -> do   (a,c) <- mputl l (b,mmissing l) 
>                                       return (a,b,c))
>
> bx2symMLens ::  Monad m => StateTBX m c a b -> 
>                 SMLens m (Maybe c) a b
> bx2symMLens bx = SMLens mputlr mputrl missing where
>  mputlr (a,ms) =  do  s <- case ms of
>                              Just s'  -> return s'
>                              Nothing  -> initl bx a
>                       (b,s') <-  (runStateT (do {  setl bx  a ; getr bx }) s)
>                       return (b,Just s')
>  mputrl (b,ms) =  do  s <- case ms of
>                              Just s'  -> return s'
>                              Nothing  -> initr bx b
>                       (a,s') <- runStateT (do {  setr bx  b ; getl bx }) s
>                       return (a,Just s')
>  missing  = Nothing

Constants

> constBX  ::    Monad tT => aT -> StateTBX tT aT () aT
> constBX a = StateTBX  (return ()) 
>                       (const (return ())) 
>                       (const (return a)) 
>                       get put return

Pairs

> fstBX :: (Monad m) => b -> StateTBX m (a,b) (a,b) a 
> fstBX b0 = StateTBX  (get)
>                      (put) 
>                      return
>                      (gets fst)
>                      (\a -> do  (_,b) <- get
>                                 put (a,b))
>                      (\a -> return (a,b0))
>
> sndBX :: Monad m => a -> StateTBX m (a,b) (a,b) b 
> sndBX a0 = StateTBX  (get)
>                      (put) 
>                      return
>                      (gets snd)
>                      (\b -> do  (a,_) <- get
>                                 put (a,b))
>                      (\b -> return (a0,b))

Products
 
> pairBX :: Monad m =>  StateTBX m s1 a1 b1 -> 
>                       StateTBX m s2 a2 b2 -> 
>                       StateTBX m (s1,s2) (a1,a2) (b1,b2)
> pairBX bx1 bx2 = StateTBX  (do  a1 <- left (getl bx1)
>                                 a2 <- right (getl bx2)
>                                 return (a1,a2))
>                            (\(a1,a2) -> do  left (setl bx1 a1)
>                                             right (setl bx2 a2))
>                            (\(a1,a2) -> do  s1 <- initl bx1 a1
>                                             s2 <- initl bx2 a2
>                                             return (s1,s2))
>                            (do  b1 <- left (getr bx1)
>                                 b2 <- right (getr bx2)
>                                 return (b1,b2))
>                            (\(b1,b2) -> do  left (setr bx1 b1)
>                                             right (setr bx2 b2))
>                            (\(b1,b2) -> do  s1 <- initr bx1 b1
>                                             s2 <- initr bx2 b2
>                                             return (s1,s2))

Sums

> inlBX :: Monad m => x -> StateTBX m (x,Maybe y) x (Either x y)
> inlBX initX = StateTBX getA setA initA getB setB initB
>   where  getA             =  do  {(x,_) <- get; return x}
>          getB             =  do  (x,my) <- get
>                                  case my of  
>                                    Just y -> return (Right y)
>                                    Nothing -> return (Left x)
>          setA x'          =  do {(x,my) <- get ; put (x',my)}
>          setB (Left x)    =  do {put (x,Nothing)}
>          setB (Right y)   =  do {(x,_) <- get; put (x,Just y)}
>          initA x          =  return (x,Nothing)
>          initB (Left x)   =  return (x,Nothing)
>          initB (Right y)  =  return (initX,Just y)
>           
> inrBX :: Monad m => y -> StateTBX m (y,Maybe x) y (Either x y)
> inrBX initY = StateTBX getA setA initA getB setB initB
>   where  getA             =  do {(y,_) <- get; return y}
>          getB             =  do  (y,mx) <- get
>                                  case mx of 
>                                    Just x -> return (Left x)
>                                    Nothing -> return (Right y)
>          setA y'          =  do {(y,mx) <- get ; put (y',mx)}
>          setB (Left x)    =  do {(y,_) <- get; put (y,Just x)}
>          setB (Right y)   =  do {put (y,Nothing)}
>          initA y          =  return (y,Nothing)
>          initB (Left x)   =  return (initY,Just x)
>          initB (Right y)  =  return (y,Nothing)
>    
> sumBX ::  Monad m =>  StateTBX m s1 a1 b1 -> 
>           StateTBX m s2 a2 b2 -> 
>           StateTBX m (Bool,s1,s2) (Either a1 a2) (Either b1 b2) 
> sumBX bx1 bx2 = StateTBX getA setA initA getB setB initB
>   where getA               = do  (b,s1,s2) <- get; 
>                                  if b then 
>                                    do  (a1,_) <- lift (runStateT (getl bx1) s1) 
>                                        return (Left a1)
>                                  else do  (a2,_) <- lift (runStateT (getl bx2) s2) 
>                                           return (Right a2) 
>         getB               = do  (b,s1,s2) <- get; 
>                                  if b then 
>                                    do  (b1,_) <- lift (runStateT (getr bx1) s1) 
>                                        return (Left b1) 
>                                  else do  (b2,_) <- lift (runStateT (getr bx2) s2) 
>                                           return (Right b2) 
>         setA (Left a1)     = do  (b,s1,s2) <- get
>                                  ((),s1') <- lift (runStateT (setl bx1 a1) s1)
>                                  put (True,s1',s2)
>         setA (Right a2)    = do  (b,s1,s2) <- get
>                                  ((),s2') <- lift (runStateT (setl bx2 a2) s2)
>                                  put (False,s1,s2')
>         setB (Left b1)     = do  (b,s1,s2) <- get
>                                  ((),s1') <- lift (runStateT (setr bx1 b1) s1)
>                                  put (True,s1',s2)
>         setB (Right b2)    = do  (b,s1,s2) <- get
>                                  ((),s2') <- lift (runStateT (setr bx2 b2) s2)
>                                  put (False,s1,s2')
>         initA (Left a1)    = do  s1 <- initl bx1 a1
>                                  return (True, s1, undefined)
>         initA (Right a2)   = do  s2 <- initl bx2 a2
>                                  return (False, undefined, s2)
>         initB (Left b1)    = do  s1 <- initr bx1 b1
>                                  return (True, s1, undefined)
>         initB (Right b2)   = do  s2 <- initr bx2 b2
>                                  return (False, undefined, s2)

List 

> listBX ::  Monad m =>  StateTBX m s a b -> 
>                        StateTBX m (Int, [s]) [a] [b]
> listBX bx = StateTBX  getL setL initL getR setR initR
>   where getL        =  do {  (n,cs) <- get; 
>                              mapM (lift . evalStateT (getl bx) ) (take n cs)}
>         getR        =  do {  (n,cs) <- get; 
>                              mapM (lift . evalStateT (getr bx) ) (take n cs)}
>         setL as     =  do {  (_,cs) <- get;
>                              cs' <- sets (setl bx) (initl bx) cs as;
>                              put (length as,cs')}
>         setR bs     =  do {  (_,cs) <- get;
>                              cs' <- sets (setr bx) (initr bx) cs bs;
>                              put (length bs,cs')}
>         initL as    =  do {  cs <- mapM (initl bx) as; 
>                              return (length as, cs) }
>         initR bs    =  do {  cs <- mapM (initr bx) bs;
>                              return (length bs, cs) }
>
>         sets set init [] []          =  return []
>         sets set init (c:cs) (x:xs)  =  do  c' <- lift (execStateT (set x) c)
>                                             cs' <- sets set init cs xs
>                                             return (c':cs')
>         sets set init cs []          =  return cs
>         sets set init [] xs          =  lift (mapM init xs)
