{-# LINE 146 "bx-effects.lhs" #-}
--  This file contains machine-readable code extracted from the paper.
--  ------------------------------------------------------------------


{-#  LANGUAGE RankNTypes, EmptyDataDecls, GADTs, FlexibleContexts, FlexibleInstances, MultiParamTypeClasses  #-}

import Control.Monad (MonadPlus, mplus, mzero, guard)
import Control.Monad.State (MonadState)
import Control.Monad.Reader (MonadReader, Reader, ask)
import Control.Monad.Writer (MonadWriter, Writer, tell)
import Control.Monad.Identity (Identity, runIdentity)
import Data.List (delete,find)
{-# LINE 280 "background.ltx" #-}
newtype StateT sT tT aT = STATET (sT -> tT (aT,sT))

instance Monad tT => Monad (StateT sT tT) where
  return a  = STATET (\ s -> return (a,s))
  m >>= k   = STATET (\ s -> do {  (a,s') <- runStateT m s ; runStateT (k a) s' })
{-# LINE 293 "background.ltx" #-}
getT :: Monad tT => StateT sT tT sT
getT = STATET (\ s -> return (s,s))

setT :: Monad tT => sT -> StateT sT tT ()
setT s' = STATET (\ s -> return ((), s'))
{-# LINE 309 "background.ltx" #-}
lift       :: Monad tT => tT aT -> StateT sT tT aT
lift m = STATET (\ s -> do { a <- m ; return (a,s) })
{-# LINE 319 "background.ltx" #-}
runStateT  :: StateT sT tT aT -> sT -> tT (aT,sT) 
runStateT (STATET f) = f
{-# LINE 354 "background.ltx" #-}
type State sT aT = StateT sT Identity aT

runState :: State sT aT -> sT -> (aT,sT)
runState m s = runIdentity (runStateT m s)
{-# LINE 364 "background.ltx" #-}
getsT :: Monad tT => (sT -> aT) -> StateT sT tT aT
getsT f = do { s <- getT ; return (f s) }

evalStateT  :: Monad tT => StateT sT tT aT -> sT -> tT aT 
evalStateT m s  = do { (a,s') <- runStateT m s ; return a }

execStateT  :: Monad tT => StateT sT tT aT -> sT -> tT sT
execStateT m s  = do { (a,s') <- runStateT m s ; return s' }
{-# LINE 493 "background.ltx" #-}
data Lens aT bT =  Lens { view    :: aT -> bT, update  :: aT -> bT -> aT}
{-# LINE 12 "monad.ltx" #-}
data BX tT aT bT = BX {  getL  :: tT aT,     setL  :: aT -> tT (),
                               getR  :: tT bT,     setR  :: bT -> tT () }    
{-# LINE 172 "monad.ltx" #-}
type StateTBX tT sT aT bT  = BX (StateT sT tT) aT bT
{-# LINE 326 "monad.ltx" #-}
data MLens tT aT bT = MLens {  mview :: aT -> bT,
                               mupdate :: aT -> bT -> tT aT }
{-# LINE 369 "monad.ltx" #-}
fstMLens :: Monad tT => MLens tT (aT,bT) aT
fstMLens = MLens mv mupd where
  mv (s1,s2)        = s1
  mupd (s1,s2) s1'  = return (s1',s2)

sndMLens :: Monad tT => MLens tT (aT,bT) bT
sndMLens = MLens mv mupd where
  mv (s1,s2)        = s2
  mupd (s1,s2) s2'  = return (s1,s2')
{-# LINE 386 "monad.ltx" #-}
idMLens :: Monad tT => MLens tT aT aT
idMLens = MLens (\ a -> a) (\ _ a -> return a)
{-# LINE 19 "compositionLR.ltx" #-}
thetaM :: Monad tT => MLens tT aT bT -> StateT bT tT cT -> StateT aT tT cT
thetaM l m = do    a <- getT ; let b = mview l  a ;
                   (c,b') <- lift (runStateT m b) ;
                   a' <- lift ((mupdate l ) a b') ;
                   setT a' ; return c 
{-# LINE 42 "compositionLR.ltx" #-}
left   ::  Monad tT => StateT s1T tT aT -> StateT (s1T, s2T) tT aT
left   =   thetaM fstMLens

right  ::  Monad tT => StateT s2T tT aT -> StateT (s1T, s2T) tT aT
right  =   thetaM sndMLens
{-# LINE 92 "compositionLR.ltx" #-}
type Join a b = (a,b)
{-# LINE 11 "equivalence.ltx" #-}
identity :: Monad tT => StateTBX tT aT aT aT
identity = BX getT setT getT setT
{-# LINE 52 "equivalence.ltx" #-}
data Iso aT bT = Iso {to :: aT -> bT, from :: bT -> aT}
inv h = Iso ((from h )) ((to h ))

iota :: Monad tT =>  Iso s1T s2T -> StateT s1T tT aT -> StateT s2T tT aT
iota h m = do {  s2 <- getT; (a,s1) <- lift (runStateT m ((from h ) s2)); 
                 setT ((to h ) s1); return a}
iotaInv h = iota (inv h)
{-# LINE 32 "examples.ltx" #-}
data InitStateTBX tT sT aT bT = InitStateTBX {
  getL'   :: StateT sT tT aT,     setL'   :: aT -> StateT sT tT (),   initL'   :: aT -> tT sT,
  getR'   :: StateT sT tT bT,     setR'   :: bT -> StateT sT tT (),   initR'   :: bT -> tT sT  }
{-# LINE 109 "examples.ltx" #-}
dual :: StateTBX tT sT aT bT -> StateTBX tT sT bT aT  
dual bx = BX (getR bx ) (setR bx ) (getL bx ) (setL bx )
{-# LINE 129 "examples.ltx" #-}
fstIBX    ::    Monad tT => bT -> InitStateTBX tT (aT,bT) (aT,bT) aT 
sndIBX    ::    Monad tT => aT -> InitStateTBX tT (aT,bT) (aT,bT) bT 
{-# LINE 134 "examples.ltx" #-}
pairBX   ::    Monad tT => StateTBX tT s1T a1T b1T -> StateTBX tT  s2T a2T b2T -> 
               StateTBX tT (s1T,s2T) (a1T,a2T) (b1T,b2T)
pairBX bx1 bx2 = BX gl sl gr sr where
      gl =  do {  a1 <- left ((getL bx1 )); a2 <- right ((getL bx2 )); return (a1,a2)}
      sl (a1,a2)  =  do {  left ((setL bx1 ) a1); right ((setL bx2 ) a2)}
      gr          =  do    {  b1 <- left ((getR bx1 )); b2 <- right ((getR bx2 )); return (b1,b2)}
      sr          =  \    (b1,b2) -> do {  left (setR bx1  b1); right (setR bx2  b2)}
{-# LINE 156 "examples.ltx" #-}
constBX  ::    Monad tT => aT -> InitStateTBX tT aT () aT
constBX a = InitStateTBX     (return ()) 
                             (const (return ())) 
                             (const (return a)) 
                             getT 
                             setT 
                             return

fstIBX b0 = InitStateTBX    (getT)
                            (setT) 
                            return 
                            (getsT fst)
                            (\ a -> do  (_,b) <- getT
                                        setT (a,b))
                            (\ a -> return (a,b0))

sndIBX a0 = InitStateTBX    (getT)
                            (setT) 
                            return
                            (getsT snd)
                            (\ b -> do  (a,_) <- getT
                                        setT (a,b))
                            (\ b -> return (a0,b))
{-# LINE 214 "examples.ltx" #-}
inlBX  ::  Monad tT => aT -> StateTBX tT (aT,Maybe bT) aT (Either aT bT) 
inrBX  ::  Monad tT => bT -> StateTBX tT (bT,Maybe aT) bT (Either aT bT)
{-# LINE 221 "examples.ltx" #-}
sumBX  ::  Monad tT => StateTBX tT s1T a1T b1T -> StateTBX tT  s2T a2T b2T -> 
           StateTBX tT (Bool,s1T,s2T) (Either a1T a2T) (Either b1T b2T) 
sumBX bx1 bx2 = BX gl sl gr sr where
         gl = do {  (b,s1,s2) <- getT ;
                    if b  then  do {  (a1,_) <- lift (runStateT (getL bx1 ) s1) ; return (Left a1) }
                          else  do {  (a2,_) <- lift (runStateT (getL bx2 ) s2) ; return (Right a2) } }
         sl (Left a1)   = do {  (b,s1,s2) <- getT ;
                                ((),s1') <- lift (runStateT ((setL bx1 ) a1) s1) ;
                                setT (True,s1',s2) }
         sl (Right a2)  = do {  (b,s1,s2) <- getT ;
                                ((),s2') <- lift (runStateT ((setL bx2 ) a2) s2) ;
                                setT (False,s1,s2') }
         gr  = do  { (b,s1,s2) <- getT ; if b then do { (b1,_) <- lift (runStateT ((getR bx1 )) s1) ; return (Left b1) } else do { (b2,_) <- lift (runStateT ((getR bx2 )) s2) ; return (Right b2) } }
         sr  = \   b1b2 -> case b1b2 of Left b1 -> do { (b,s1,s2) <- getT ; ((),s1') <- lift (runStateT (setR bx1  b1) s1) ; setT (True,s1',s2) } ; Right b2 -> do { (b,s1,s2) <- getT ; ((),s2') <- lift (runStateT (setR bx2  b2) s2) ; setT (False,s1,s2') } 
{-# LINE 248 "examples.ltx" #-}
inlBX a0 = BX gl sl gr sr
  where  gl             =  do  {(x,_) <- getT; return x}
         gr             =  do  (x,my) <- getT
                               case my of  
                                 Just y -> return (Right y)
                                 Nothing -> return (Left x)
         sl x'          =  do {(x,my) <- getT ; setT (x',my)}
         sr (Left x)    =  do {setT (x,Nothing)}
         sr (Right y)   =  do {(x,_) <- getT; setT (x,Just y)}

inrBX b0 = BX gl sl gr sr
  where  gl             =  do {(y,_) <- getT; return y}
         gr             =  do  (y,mx) <- getT
                               case mx of 
                                 Just x -> return (Left x)
                                 Nothing -> return (Right y)
         sl y'          =  do {(y,mx) <- getT ; setT (y',mx)}
         sr (Left x)    =  do {(y,_) <- getT; setT (y,Just x)}
         sr (Right y)   =  do {setT (y,Nothing)}

inlBX'  ::  Monad tT => aT -> InitStateTBX tT (aT,Maybe bT) aT (Either aT bT) 
inlBX' a0 = InitStateTBX gl sl il gr sr ir
  where  gl             =  do  {(x,_) <- getT; return x}
         gr             =  do  (x,my) <- getT
                               case my of  
                                 Just y -> return (Right y)
                                 Nothing -> return (Left x)
         sl x'          =  do {(x,my) <- getT ; setT (x',my)}
         sr (Left x)    =  do {setT (x,Nothing)}
         sr (Right y)   =  do {(x,_) <- getT; setT (x,Just y)}
         il a           =  return (a,Nothing)
         ir (Left a)    =  return (a,Nothing)
         ir (Right b)   =  return (a0,Just b)

inrBX'  ::  Monad tT => bT -> InitStateTBX tT (bT,Maybe aT) bT (Either aT bT)
inrBX' b0 = InitStateTBX gl sl il gr sr ir
  where  gl             =  do {(y,_) <- getT; return y}
         gr             =  do  (y,mx) <- getT
                               case mx of 
                                 Just x -> return (Left x)
                                 Nothing -> return (Right y)
         sl y'          =  do {(y,mx) <- getT ; setT (y',mx)}
         sr (Left x)    =  do {(y,_) <- getT; setT (y,Just x)}
         sr (Right y)   =  do {setT (y,Nothing)}
         il b           =  return (b,Nothing)
         ir (Left a)    =  return (b0,Just a)
         ir (Right b)   =  return (b,Nothing)

sumBX'  ::  Monad tT => InitStateTBX tT s1T a1T b1T -> InitStateTBX tT  s2T a2T b2T -> 
           InitStateTBX tT (Bool,s1T,s2T) (Either a1T a2T) (Either b1T b2T) 
sumBX' bx1 bx2 = InitStateTBX gl sl il gr sr ir where
         gl = do {  (b,s1,s2) <- getT ;
                    if b  then  do {  (a1,_) <- lift (runStateT (getL' bx1 ) s1) ; return (Left a1) }
                          else  do {  (a2,_) <- lift (runStateT (getL' bx2 ) s2) ; return (Right a2) } }
         sl (Left a1)   = do {  (b,s1,s2) <- getT ;
                                ((),s1') <- lift (runStateT ((setL' bx1 ) a1) s1) ;
                                setT (True,s1',s2) }
         sl (Right a2)  = do {  (b,s1,s2) <- getT ;
                                ((),s2') <- lift (runStateT ((setL' bx2 ) a2) s2) ;
                                setT (False,s1,s2') }
         il (Left a1)   = do {  s1 <- initL' bx1  a1; return (True, s1, undefined) }
         il (Right a2)  = do {  s2 <- initL' bx2  a2; return (False, undefined, s2) }
         gr  = do  { (b,s1,s2) <- getT ; if b then do { (b1,_) <- lift (runStateT ((getR' bx1 )) s1) ; return (Left b1) } else do { (b2,_) <- lift (runStateT ((getR' bx2 )) s2) ; return (Right b2) } }
         sr  = \   b1b2 -> case b1b2 of Left b1 -> do { (b,s1,s2) <- getT ; ((),s1') <- lift (runStateT (setR' bx1  b1) s1) ; setT (True,s1',s2) } ; Right b2 -> do { (b,s1,s2) <- getT ; ((),s2') <- lift (runStateT (setR' bx2  b2) s2) ; setT (False,s1,s2') } 
         ir  = \  b1b2 -> case b1b2 of Left b1 -> do {s1 <- initR' bx1 b1; return (True, s1, undefined)} ; Right b2 -> do {s2 <- initR' bx2 b2; return (False, undefined, s2)}
{-# LINE 337 "examples.ltx" #-}
listIBX ::  Monad tT => 
            InitStateTBX tT sT aT bT -> InitStateTBX tT (Int,[sT]) [aT] [bT]
listIBX bx = InitStateTBX gl sl il gr sr ir where 
        gl        =  do {  (n,cs) <- getT; mapM (lift . evalStateT (getL' bx ) ) (take n cs) }
        sl as     =  do {  (_,cs) <- getT;
                           cs' <- lift (sets (execStateT . (setL' bx )) (initL' bx ) as cs);
                           setT (length as,cs')}
        il as     =  do {  cs <- mapM ((initL' bx )) as; return (length as, cs) }
        gr        =  do  {  (n,cs) <- getT; mapM (lift . evalStateT (getR' bx ) ) (take n cs)}
        sr bs     =  do  {  (_,cs) <- getT; cs' <- lift (sets (execStateT . (setR' bx )) (initR' bx ) bs cs); setT (length bs,cs')}
        ir bs     =  do  {  cs <- mapM ((initR' bx )) bs; return (length bs, cs)}
{-# LINE 365 "examples.ltx" #-}
sets ::  Monad tT => (aT -> cT -> tT cT) -> (aT -> tT cT) -> [aT] -> [cT] -> tT [cT]
sets set init []      cs      =  return cs
sets set init (x:xs)  (c:cs)  =  do {  c' <- set x c ; cs' <- sets set init xs cs ;
                                       return (c':cs') }
sets set init xs      []      =  mapM init xs
{-# LINE 414 "examples.ltx" #-}
switch ::  (cT -> StateTBX (Reader cT) sT aT bT) -> StateTBX (Reader cT) sT aT bT
switch f = BX gl sl gr sr where
  gl          =  do {c <- lift ask; getL (f c) }
  sl a        =  do {c <- lift ask; setL (f c)  a}
  gr          =  do  {c <- lift ask; getR (f c) }
  sr          =  \   b -> do {c <- lift ask; setR (f c)  b} 
{-# LINE 444 "examples.ltx" #-}
frac a b = a/b
{-# LINE 466 "examples.ltx" #-}
invBX :: StateTBX Maybe Float Float Float
invBX = BX getT setL (getsT (\ a -> frac 1 a)) setR where
  setL a      =  do {lift (guard (a /= 0)); setT a}
  setR b      =  do {lift (guard (b /= 0)); setT (frac 1 b)}
{-# LINE 476 "examples.ltx" #-}
readSomeBX ::  (Read aT, Show aT) => StateTBX Maybe (aT,String) aT String
readSomeBX = BX (getsT fst) setL (getsT snd) setR where
  setL a'  = setT (a',show a')
  setR b'  = do {  (_,b) <- getT ;
                   if b==b' then return () else case reads b of
                       (( a',""):_)  -> setT (a',b)
                       _             -> lift Nothing }
{-# LINE 492 "examples.ltx" #-}
partialBX ::  Monad tT => (forall aT. tT aT) -> (aT -> Maybe bT) -> (bT -> Maybe aT) -> 
              StateTBX tT (aT,bT) aT bT
partialBX err f g = BX (getsT fst) setL (getsT snd) setR where
  setL a'  =  case f a' of  Just b'  -> setT (a',b')
                            Nothing  -> lift err
  setR b'  =  case g b' of  Just a'  -> setT (a',b')
                            Nothing  -> lift err
{-# LINE 556 "examples.ltx" #-}
nondetBX ::  (aT->bT->Bool) -> (aT->[bT]) -> (bT->[aT]) -> 
             StateTBX [] (aT,bT) aT bT
nondetBX ok bs as = BX (getsT fst) setL (getsT snd) setR where
  setL a'  = do {  (a,b) <- getT ;
                   if ok a' b then setT (a',b) else
                     do { b' <- lift (bs a') ; setT (a',b') } }
  setR b'  = do {  (a,b) <- getT ;
                   if ok a b' then setT (a,b') else
                     do { a' <- lift (as b') ; setT (a',b') } }
{-# LINE 597 "examples.ltx" #-}
signalBX ::  (Eq aT, Eq bT, Monad tT) => (aT -> tT ()) -> (bT -> tT ()) -> 
             StateTBX tT sT aT bT -> StateTBX tT sT aT bT
signalBX sigA sigB bx = BX ((getL bx )) sl ((getR bx )) sr where
  sl a'   = do {  a <- getL bx ; setL bx  a';
                  lift (if a /= a' then sigA a' else return ()) }
  sr b'   = do {  b <- getR bx  ; setR bx  b' ;
                  lift (if b /= b' then sigB b' else return ()) }    
{-# LINE 619 "examples.ltx" #-}
alertBX ::  (Eq aT, Eq bT) => StateTBX IO sT aT bT -> StateTBX IO sT aT bT
alertBX = signalBX  (\ _ -> putStrLn "Left") (\ _ -> putStrLn "Right")
{-# LINE 632 "examples.ltx" #-}
logBX    ::  (Eq aT, Eq bT) =>  StateTBX (Writer [Either aT bT]) sT aT bT -> 
                                StateTBX (Writer [Either aT bT]) sT aT bT
logBX bx = signalBX (\a -> tell [Left a]) (\b -> tell [Right b]) bx
{-# LINE 712 "examples.ltx" #-}
dynamicBX ::  (Eq aT, Eq bT, Monad tT) => 
              (aT -> bT -> tT bT) -> (aT -> bT -> tT aT) -> 
              StateTBX tT ((aT,bT),[((aT,bT),bT)],[((aT,bT),aT)]) aT bT
dynamicBX f g = BX (getsT (fst.fst3)) setL (getsT (snd.fst3)) setR where
         setL a' = do {  ((a,b),fs,bs) <- getT ;
                         if a==a' then return () else
                           case lookup (a',b) fs of
                             Just b'  -> setT ((a',b'), ((a',b),b') : fs, bs) 
                             Nothing  -> do {  b' <- lift (f a' b) ; 
                                               setT ((a',b'), ((a',b),b') : fs, bs) } }
         setR b' = do   {((a,b),fs,bs) <- getT ; if b==b' then return () else do  { a' <- case lookup (a,b') bs of { Just a'  -> return a' ; Nothing  -> lift (g a b') } ; setT ((a',b'), fs, ((a,b'),a') : bs) } } 
{-# LINE 727 "examples.ltx" #-}
fst3 (a,_,_)  = a
snd3 (_,b,_)  = b
thd3 (_,_,c)  = c
{-# LINE 759 "examples.ltx" #-}
dynamicIOBX ::  (Eq aT, Eq bT, Show aT, Show bT, Read aT, Read bT) => 
                StateTBX IO ((aT,bT), [((aT,bT),bT)], [((aT,bT),aT)]) aT bT
dynamicIOBX = dynamicBX matchIO (flip matchIO)

matchIO :: (Show aT, Show bT, Read bT) => aT -> bT -> IO bT
matchIO a b = do {  putStrLn ("Setting " ++ show a) ;
                    putStr ("Replacement for " ++ show b ++ "?") ;
                    s <- getLine ; return (read s) }
{-# LINE 790 "examples.ltx" #-}
dynamicSearchBX ::  
  (Eq aT, Eq bT, Enum aT, Bounded aT, Enum bT, Bounded bT) =>
  (aT -> bT -> Bool) -> 
  StateTBX Maybe ((aT,bT), [((aT,bT),bT)], [((aT,bT),aT)]) aT bT
dynamicSearchBX p = dynamicBX (search p) (flip (search (flip p)))

search :: (Enum bT, Bounded bT) => (aT->bT->Bool) -> aT -> bT -> Maybe bT
search p a _ = find (p a) [minBound..maxBound]
{-# LINE 18 "symmetric.ltx" #-}
data SLens cT aT bT = SLens {  putR     :: (aT,cT) -> (bT,cT),
                               putL     :: (bT,cT) -> (aT,cT),
                               missing  :: cT }
{-# LINE 25 "symmetric.ltx" #-}
type StateBX sT aT bT      = StateTBX Identity sT aT bT
{-# LINE 168 "symmetric.ltx" #-}
data SMLens0 tT cT aT bT = SMLens0 {  mputR  :: (aT,cT) -> tT (bT,cT),
                                      mputL  :: (bT,cT) -> tT (aT,cT),
                                      mmissing :: cT }
{-# LINE 221 "symmetric.ltx" #-}
compSMLens ::  Monad tT =>  SMLens0 tT s1T aT bT -> SMLens0 tT s2T bT cT -> SMLens0 tT (s1T,s2T) aT cT
sl1 `compSMLens` sl2 = SMLens0 putR putL missing where
        putR (a,(s1,s2)) = do {  (b,s1') <- mputR sl1 (a,s1);
                                 (c,s2') <- mputR sl2 (b,s2);
                                 return (c,(s1',s2')) }
        putL = \   (c, (s1,s2)) ->  do {  (b,s2') <- mputL sl2 (c,s2);  (a,s1') <- mputL sl1 (b,s1);  return (a,(s1',s2')) } 
        missing = (mmissing sl1 , mmissing sl2 )
{-# LINE 420 "symmetric.ltx" #-}
smlens2bx sl = InitStateTBX (getsT fst3) setL initL (getsT snd3) setR initR where
  setL a'  = do {  c <- getsT thd3 ; (b',c') <- lift ((mputR sl  (a',c))) ; setT (a',b',c') }
  setR b'  = do {  c <- getsT thd3; (a',c') <- lift ((mputL sl  (b',c))) ; setT (a',b',c') }  
  initL a  = do { (b,c) <- mputR sl  (a,mmissing sl ) ; return (a,b,c) }
  initR b  = do { (a,c) <- mputL sl  (b,mmissing sl ) ; return (a,b,c) }
{-# LINE 434 "symmetric.ltx" #-}
fail :: SMLens0 Maybe () () ()
fail = SMLens0 m m () where m _ = Nothing
{-# LINE 609 "symmetric.ltx" #-}
bx2smlens bx = SMLens0 mputR mputL mmissing where
  mputR (a,Just s)   =  do  { (b,s') <- runStateT (do { setL' bx  a ; getR' bx  }) s ; return (b, Just s') }
  mputR (a,Nothing)  =  do  { (b,s') <- (runL bx )  ((getR' bx )) a ; return (b,Just s') }
  mputL              = \  (b,mc) -> case mc of Just s -> do {(a,s') <- runStateT (do { setR' bx  b ; getL' bx  }) s ; return (a, Just s') } ; Nothing ->  do { (b,s') <- (runR bx ) ((getL' bx )) b ; return (b,Just s') }
  mmissing            = Nothing  
{-# LINE 617 "symmetric.ltx" #-}
runL ::  Monad tT => InitStateTBX tT sT aT bT -> StateT sT tT cT -> aT -> tT (cT,sT)
runL bx m a = do { s <- initL' bx  a ; runStateT m s }
{-# LINE 623 "symmetric.ltx" #-}
runR ::  Monad tT => 
         InitStateTBX tT sT aT bT -> StateT sT tT cT -> bT -> tT (cT,sT)
runR bx m a = do { s <- initR' bx  a ; runStateT m s }
{-# LINE 781 "symmetric.ltx" #-}
data InitStateBX sT aT bT = 
  InitStateBX {  igetL :: State sT aT, isetL :: aT -> State sT (), initL :: aT -> sT,
                 igetR :: State sT bT, isetR :: bT -> State sT (), initR :: bT -> sT}
{-# LINE 788 "symmetric.ltx" #-}
bx2slens ::  InitStateBX sT aT bT -> SLens (Maybe sT) aT bT
bx2slens bx = SLens putR putL missing where
  putR (a,Just s)    =  let (b,s') = runState (do { isetL bx  a ; igetR bx  }) s in (b, Just s')
  putR (a,Nothing)   =  let (b,s') = runState ((igetR bx )) ((initL bx  a)) in (b,Just s')
  putL               = \  (b,mc) -> case mc of Just s -> let (a,s') = runState (do { isetR bx  b ; igetL bx  }) s in (a, Just s')  ; Nothing ->  let (b,s') = runState ((igetL bx )) ((initL bx  b)) in (b,Just s') 
  missing            = Nothing

slens2bx :: SLens cT aT bT -> InitStateBX (aT,bT,cT) aT bT
slens2bx sl = InitStateBX igetL isetL initL igetR isetR initR where
  igetL     = getsT fst3
  igetR     = getsT snd3
  isetL a'  = do {  c <- getsT thd3; let (b',c') = (putR sl  (a', c)) in setT (a',b',c') }
  isetR = \ b' -> do {  c <- getsT thd3 ; let (a',c') = (putL sl  (b', c)) in setT (a',b',c') } 
  initL a = let (b,c) = putR sl  (a,missing sl ) in (a,b,c)
  initR = \ b -> let (a,c) = putL sl  (b,missing sl ) in (a,b,c) 
{-# LINE 263 "bx-effects.lhs" #-}
main = return 0
